"""
Gaussian (GA) Radial Basis function class

import suggestion: from gax import *

"""

from scipy import exp, zeros, shape
from rbfClassX import rbfx

from scipy.linalg import qr, solve_triangular
from numpy import dot, zeros, diag
from scipy.special import eval_hermite, gamma
from math import sqrt, log
from scipy.special import eval_genlaguerre, factorial

class gax(rbfx):

	def rbf(self,r,s):
		return exp(-s**2*r**2)

	def D1(self, r, s, x):
		return -2*s**2*x*exp(-r**2*s**2)

	def D2(self, r, s, x):
		return s**2*(4*s**2*x**2 - 2)*exp(-r**2*s**2)

	def D3(self, r, s, x):
		return -4*s**4*x*(2*s**2*x**2 - 3)*exp(-r**2*s**2)

	def D4(self, r, s, x):
		return s**4*(16*s**4*x**4 - 48*s**2*x**2 + 12)*exp(-r**2*s**2)

	def div(self, r, s, x, y=0, z=0):
		return -2*s**2*(x + y + z)*exp(-r**2*s**2)

	def L(self, r, s):
		return 4*s**2*(s**2*r**2  - 1)*exp(-r**2*s**2)

	def B(self, r, s, x, y):
		return 16*s**4*(s**4*x**4 + 2*s**4*x**2*y**2 + s**4*y**4 - 4*s**2*x**2 - 4*s**2*y**2 + 2)*exp(-r**2*s**2)

	def D12(self,r,s,x,y):
		return -4*s**4*x*(2*s**2*y**2 - 1)*exp(-r**2*s**2)

	def D22(self,r,s,x,y):
		return s**4*(16*s**4*x**2*y**2 - 8*s**2*x**2 - 8*s**2*y**2 + 4)*exp(-r**2*s**2)



# ---------------------- hyperviscosity -----------------------------------

	
	def hyperViscosity(obj,d,k,s,r):
		return ((-1.0)**k)*s**(2*k)*(-4.0)**k*factorial(k)*eval_genlaguerre(k, d/2 - 1,(s*r)**2)*exp(-(s*r)**2)
	

			
