"""
Example of interpolation on the surface of a shere.

run interp3d.py

"""

from numpy import inf
from numpy.linalg import norm, cond
import rbfCenters as rc

from iqx import *
phi = iqx() 
s = 2.5

N, M = 6000, 7000

xc, yc, zc = rc.sphereCenters(N)             # centers
x, y, z = rc.sphereCenters(M,ch=1)           # evaluation points

from F3a import *
fn = F3a()

f = fn.F(xc,yc,zc)   # function values at the center locations
fe = fn.F(x,y,z)     # exact function values at the evaluation points

r, _, _, _ = phi.distanceMatrix3d(xc,yc,zc)
B = phi.rbf(r,s)
print('kappa(B): {:1.2e}'.format(cond(B)))

mu = 5e-15
safe = False
a = phi.solve(B,f,mu,safe)    # expansiion coefficients

re, _, _, _ = phi.distanceMatrix3d(xc,yc,zc,x,y,z)
H = phi.rbf(re,s)

fa = H@a       # evaluate the interpolant

er = norm( fa - fe, inf)

print('3d interpolation error: {:1.2e}'.format(er))
