"""
Example of 2d rational RBF interpolation.

reference: S. A. Sarra and Y. Bai. A rational radial basis function method for accurately 
           resolving discontinuities and steep gradients. Applied Numerical Mathematics 130, 
           p. 131-142, 2018.  DOI: 10.1016/j.apnum.2018.04.001

usage: run interp2dRRBF.py
"""

from numpy import linspace, inf, arctan, sqrt, meshgrid
import matplotlib.pyplot as plt
from numpy.linalg import norm
from iqx import *
import rbfRational as rat
import rbfCenters as rc

phi = iqx()
s = 8.0                 # shape parameter
N = 5000                # number of centers

xc, yc = rc.rectangleCenters(N, ch=3)
x = linspace(0,1,60)
x, y = meshgrid(x,x)   # 3600 iniformly spaced evaluation points
x, y = x.flatten(), y.flatten()

# --------------- the function being interpolated -------------------------

x0, y0 = 1.5, 0.25              # function parameters
r, _, _ = phi.distanceMatrix2d(xc,yc) 
re, _, _ = phi.distanceMatrix2d(xc,yc,x,y)

def F(x,y): return arctan( 125*( sqrt( (x - x0)**2 + (y - y0)**2 ) - 0.92 ) )
f, fe = F(xc,yc), F(x,y)
f = f.flatten()
# -------------------------------------------------------------------------

ra = rat.interpolate2d(phi,s,f,xc,x,yc,y)

er = norm( ra.flatten() - fe.flatten(), inf)
print('ratrional 2d interpolation error: {:1.2e}'.format(er))


from mpl_toolkits.mplot3d import Axes3D
import matplotlib.tri as tri
triang = tri.Triangulation(x,y)
fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')

ax.plot_trisurf(triang,ra)
plt.show()
