"""
Rational RBF and standard RBF approximation of a discontinuous step function.

reference: S. A. Sarra and Y. Bai. A rational radial basis function method for accurately 
           resolving discontinuities and steep gradients. Applied Numerical Mathematics 130, 
           p. 131-142, 2018.  DOI: 10.1016/j.apnum.2018.04.001

usage: run stepFunctionRRBF.py

"""


import numpy as np
import matplotlib.pyplot as plt
from iqx import *

def stepFunction(x):
	conditions = [x < 0,  x >= 0]
	functions = [lambda x: -1.0, lambda x: 1.0]
	return np.piecewise(x, conditions, functions)

phi = iqx()
mu = 0

M, N, s = 201, 80, 3.5
# M, N, s = 201, 10, 0.45

xc, x = np.linspace(-1,1,N), np.linspace(-1,1,M)
f, fe = stepFunction(xc), stepFunction(x)

# -------- standardard RBF method -----------------------------------------

r, re = phi.distanceMatrix1d(xc), phi.distanceMatrix1d(xc,x)
B, H = phi.rbf(r,s), phi.rbf(re,s)
a = phi.solve(B,f,mu,False);
fs = H@a

# -------- rational RBF method --------------------------------------------

import rbfRational as rat
ra = rat.interpolate1d(phi,s,f,xc,x,mu)

# -------------------------------------------------------------------------

plt.plot(x,fs,'b',x,fe,'r--',x,ra,'g')
plt.show()
