"""
Times the standard versus centro algorithms for calculating the eigenvalues
of a centrosymmetric matrix.

reference: S. A. Sarra. Radial Basis Function Methods - the case of symmetric domains.  
    Numerical Methods for Partial Differential Equations, 2018. DOI:10.1002/num.22272 

usage: run centroEwBench.py

"""

import matplotlib.pyplot as plt
from numpy import zeros, linspace, arange
from numpy.random import rand
from numpy.linalg import eigvals
from iqx import *
import rbfCentro as cn
import time
from scipy.linalg import eigvals

plt.close('all')

phi = iqx()
s = 230
mu = 0
its = 5
Nv = arange(600,4001,200)
Ns = len(Nv)
cTime, ncTime = zeros((Ns,)), zeros((Ns,))

for k in range(Ns):
	N = Nv[k]
	Nh = int(N/2)
	x = linspace(-100,100,N)
	f = rand(N,1)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x[:Nh],x)     # construct half-sized distance matrix
		B = phi.rbf(r,s)                       # half-sized system matrix
		ewC = cn.centroEigenvalues(B)
	cTime[k] = time.time() - startTime 
	
for k in range(Ns):
	N = Nv[k]
	x = linspace(-100,100,N)
	f = rand(N,1)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x)     
		B = phi.rbf(r,s)                       
		ewS = eigvals(B)
	ncTime[k] = time.time() - startTime 
	

plt.plot(Nv,cTime/ncTime,'b*') #,Nv,ones((Ns,)),'g--')
plt.xlabel('N'), plt.ylabel('execution time ratio')
plt.show()

